      program COU3D

*     generic Fortran 8x version: M Hopkins 7/31/90.
*     Wall driven Couette Flow, hard particle, overlap model
*     Half-spheres mounted on flat walls. Any s/D is allowed
*     since the flat wall is present.  Technically this
*     simulation is equipped to handle non-uniform diameters.
*     However, the search limits in search would need adjustment.

C Modified to include statistics of particles colliding with
C the bottom wall. Checking 'Boundary Conditions for Rapid
C Granular Flows: Flat, Frictional Walls', Jenkins (1990).
C Michel Louge 8/8/1990.

C This version contains different friction for the wall and for the
C interior or boundary particles. Note that the EXP files contain
C one more line as a result.
C It also calculates both the translation and rotation
C temperatures. MYL 10/11/90.

C This version simulates flat or bumpy walls. It fixes the
C position of the top wall also. MYL 10/13/90. It permits the
C top wall particles to move on the y=ylen palne and collide
C independently from the interior particles with no dissipation
C of the original temperature. Thus the top row of wall particles
C act as a thermostat. MYL 10/18/1990.

C Modified 9/27/90 to include full statistics in MOMENTS.

C This version has simplified outputs (no pexp file);
C the rexp file is written nevertheless
C see output subroutine for details. MYL 9/23/93.
C The moments subroutine is simplified accordingly, and it
C permits much more accurate calculations of the
C particle volume fraction at y=d/2. MYL 12/28/92.

C This version permits the specification of beta0 for the
C particles and the wall (tangential restitution). MYL 1/12/93
C
C This version also permits the calculation of the equation of
C state with fluctuating velocities in the direction 2 before the
C collision. MYL 2/8/93.
C
C------------------------------
C Note!!! The exp files must be compatible with the specification
C of beta0 and beta0w in the input and output files!!
C In addition, because the value of w may become very large especially
C as beta0w is near one, the exp files are written in exponential format
C in this version... MYL 1/16/93
C------------------------------
C
C Modified 9/16/93 to output the spin and the velocity gradient near
C the wall. This is to check Jim's calculation of the spin
C boundary condition.
C
C Modified further on 9/17/93 to carry out "center averages" i.e.,
C averages that ascribe the entire particle to the strip where its
C center is located.
C
C Modified 10/7/93 to calculate the nature of the collisions
C (sticking or slipping) across the flow.
C
C Modified 12/20/93 to output dimensionless profiles in the rexp
C files.
C
C Modified 1/19/94 to output a slightly more accurate value of the
C peak center volume fraction near the wall.
C
C Modified 1/24/94 to output all information at the first and
C second peaks of the center statistics volume fraction. Profiles
C of strip statistics are output as well as the profile
C of center statistics of volume fraction.
C
C Modified 3/20/94 to calculate various collision rates of
C change having to do with different ways of evaluating
C the dissipation at the bottom wall.
C
C Modified 6/21/94 to calculate all new quantities from
C the velocities of the incoming spheres. Correct flux signs.
C
C Modified 8/1/94 to check the relations for the flux in
C the High Friction/All Sliding (all stick regime).
C

        include "cou3fbbasccdpDcom.f"

      Integer*2 scount

********************************************************************

C
C The line below identifies the program version in the output file:
C MYL 1/12/93

        programname = "cou3fbbasccdpD3.f-8/1/94"

C "cou3fbbasccd" means  couette, 3D, flat, bumpy, beta,
C                       equation of state, spin, center,
C                       collision, dimensionless profiles,
C                       peak location & nu ctr,
C                       dissipation & collision integrals,
C			3 = version 3

      Call Initial
      scount = nsearch

CCCCCC
C The main program proceeds until the number of collisions with both
C walls is at least WALLCOLL and until the total DURATION required
C has been achieved, whichever comes later:
CCCCCC

      Do While (ncol.le.wallcoll.or.time-time0.le.duration)
         if (scount .eq. nsearch) then
            Call Grdfil
            Call Search
            scount = 0
         endif
         Call Interaction
         Call MoveSpheres

         if (nstat .ge. nsta) Call Moments
         If (ncalt .ge. ncal) Call Calibrate

         scount = scount+1
      End do

      Call Output

      Stop
      End


      Subroutine Calibrate

C     Calibrate the time step based on the overlap tolerance

      include "cou3fbbasccdpDcom.f"

      if (tolerance .ne. 1.0) dt = dt*
     &   (tolerance*aveD*min(ncalw/overw,ncalp/over))

        if (flag) then

      write (*,'(a,f10.2,2(A,I10),A,F10.6)')
     & 'T= ',time,' tot no coll',nco,' bot wall',ncol,' dt= ',dt

        else

      open (1, file='status', form='formatted', status='old')

      write (1,'(1x,A,F9.2)') 'restitution wall   ',ew
      write (1,'(1x,A,F9.2)') 'restitution disc   ',ed
      write(1,'(1x,A,F9.2)')  'friction particles ',mu
      write(1,'(1x,A,F9.2)')'friction flat wall ',muw
      write(1,'(1x,A,F9.2)')  'beta0 particles    ',beta0
      write(1,'(1x,A,F9.2,/)')'beta0 wall         ',beta0w

      write(1,'(1x,A,F9.4,/)')'u/sqrt(T)          ',
     &(v1(Nt+2)-v1(Nt+1))/sqrt((Tb1+Tb3)/2.)

      write (1,'(1x,A,F9.2)') 'time               ',time
      write(1,'(1x,A,F9.5,/)')'time step          ',dt

      write (1,'(1x,A,i10)')  'total no. of coll. ',nco
      write (1,'(1x,A,i10)')  'no of bottom coll. ',ncol


      Close(1)

        end if

      over = 0.0
      overw = 0.0
      ncalt = 0
      ncalw = 0
      ncalp = 0

      Return
      End

*******************************************************************

      Subroutine Collide(nn,nh,n1,n2,n3,dx,dy,dz,vnhn)

C     Hard particle dynamics, Impulse version. 7/31/90.

      include "cou3fbbasccdpDcom.f"

      Integer*2 nn,nh,k
      Real*8 dx,dy,dz,n1,n2,n3,t1,t2,t3
      Real*8 vnhn,vnht,vnht1,vnht2,vnht3
      Real*8 Pn,Pt,Ptns,Pts,Px,Py,Pz,Tx,Ty,Tz,ee,coef,mm,beta
      Real*8 v1b,v2b,v3b,w1b,w2b,w3b
      real*8 ycontact

      if (vnhn .gt. vnhnmx) vnhnmx = vnhn

C The coefficient of fricton is specified for the interior or
C wall particles as MU. The flat wall coefficient of friction
C is MUW. The tangential restitution coefficients are beta0
C and beta0w for the interior particles and the wall,
C respectively:

        if (nn .le.Nt) then
          mm = mu
          beta = beta0
        else
          mm = muw
          beta = beta0w
        end if

      if (vnhn .gt. 1.0e-08*vnhnmx) then
        if (nn .le. Nt) then
          ee = ed
        else
          ee = ew
        end if
      else
        ee = 1.0
      end if

C
C In collisions among the top wall particles, the coefficient of
C restitution is one, the coefficient of friction is
C zero, and beta0 is zero to avoid dissipating the
C thermostat's temperature:
C
                if (nl+nu.ne.0) then

        if ((nn-nt)*(nn-n-1).le.0.and.(nh-nt)*(nh-n-1).le.0) then

                mm=0.0
                ee=1.0
                beta = 0.0

        end if

                end if

      coef = 1.0/(1.0/mass(nn)+1.0/mass(nh))
      Pn = coef*(1.0+ee)*vnhn

      If (mm .gt. 0.0) then
C
C All collisions except those among top boundary particles:
C

          vnht1 = v1(nn)-v1(nh)-r(nn)*(n2*w3(nn)-n3*w2(nn))
     &                         -r(nh)*(n2*w3(nh)-n3*w2(nh))-n1*vnhn
          vnht2 = v2(nn)-v2(nh)-r(nn)*(n3*w1(nn)-n1*w3(nn))
     &                         -r(nh)*(n3*w1(nh)-n1*w3(nh))-n2*vnhn
          vnht3 = v3(nn)-v3(nh)-r(nn)*(n1*w2(nn)-n2*w1(nn))
     &                         -r(nh)*(n1*w2(nh)-n2*w1(nh))-n3*vnhn

C Note that the flat walls or the interior particles do not rotate,
C nor do they have a velocity in the y or z directions, so the statement
C above is equivalent to the following for wall or boundary particles:

C          vnht1 = v1(nn)-v1(nh)-r(nh)*(n2*w3(nh)-n3*w2(nh))-n1*vnhn
C          vnht2 =       -v2(nh)-r(nh)*(n3*w1(nh)-n1*w3(nh))-n2*vnhn
C          vnht3 = v3(nn)-v3(nh)-r(nh)*(n1*w2(nh)-n2*w1(nh))-n3*vnhn

        vnht = sqrt(vnht1*vnht1+vnht2*vnht2+vnht3*vnht3)

        If (vnht .ne. 0.0) then
          t1 = vnht1/vnht
          t2 = vnht2/vnht
          t3 = vnht3/vnht
        Else
          t1 = 0.0
          t2 = 0.0
          t3 = 0.0
        End If

        Ptns = 2.0*coef*vnht/7.0*(1.0+beta)
        Pts  = mm*Pn

        if (Pts .lt. Ptns) then
          Pt = Pts
        else
          Pt = Ptns

CCCCC
C The following lines can be added to check changes with '0:
C
C         if (nn.gt.nt) then
C         write(*,'(a,e12.6,a)') 'Ptns = ',Ptns,'  wall'
C         else
C         write(*,'(a,e12.6)') 'Ptns = ',Ptns
C         end if
C
CCCCC

        end if

CCCCC
C Increment the nature of the collision (stick or slip) at the
C location of the point of contact. Exclude collisions with the
C flat walls.

        if (nn.le.Nt) then

        ycontact = (y(nh)+y(nn))/2.

        k = int(ycontact/strip) + 1

                if (k.le.kcrit) then

                        if (Pts.lt.Ptns) then

                        slipcol(k) = slipcol(k)+1

                        else

                        stickcol(k) = stickcol(k)+1

                        end if
                end if
        end if

      Else
C
C Collisions among top boundary particles:
C
        Pt = 0.0

      End If

      Px = n1*Pn+t1*Pt
      Py = n2*Pn+t2*Pt
      Pz = n3*Pn+t3*Pt

CCC
C Before calculating the particle velocity following a collision with the
C bottom wall, save the particle velocity before the collision:
CCC

        if (nn.eq.nt+1) then

        v1b = v1(nh)
        v2b = v2(nh)
        v3b = v3(nh)

        w1b = w1(nh)
        w2b = w2(nh)
        w3b = w3(nh)

        end if

      v1(nh) = v1(nh)+Px/mass(nh)
C
C The following is a redundant statement to make sure that there
C is no component of velocity in the y-direction for boundary
C particles.
C
        if (nh.ge.n+1) then
      v2(nh) = 0.0
        else
      v2(nh) = v2(nh)+Py/mass(nh)
        end if

      v3(nh) = v3(nh)+Pz/mass(nh)

C The following statement makes sure that the boundary particles
C do not rotate:

      If (mm .gt. 0.0.and.nh.le.n) then
        Tx = 2.5*(n2*Pz-n3*Py)
        Ty = 2.5*(n3*Px-n1*Pz)
        Tz = 2.5*(n1*Py-n2*Px)
        w1(nh) = w1(nh)-Tx/(r(nh)*mass(nh))
        w2(nh) = w2(nh)-Ty/(r(nh)*mass(nh))
        w3(nh) = w3(nh)-Tz/(r(nh)*mass(nh))
      End If

C The following IF statements insure that the boundary particles
C are only affected by collisions among themselves.

C The next IF excludes collisions with the flat walls, but
C allows collisions among interior particles or collisions among
C wall particles:

      If (nn .le. Nt) then

C Because nh<nn, the next IF statement represents collisions
C among interior particles, which undergo changes in both velocity
C and angular velocity:

        if(nn.le.n) then

        v1(nn) = v1(nn)-Px/mass(nn)
        v2(nn) = v2(nn)-Py/mass(nn)
        v3(nn) = v3(nn)-Pz/mass(nn)

                If (mm .gt. 0.0) then
          w1(nn) = w1(nn)-Tx/(r(nn)*mass(nn))
          w2(nn) = w2(nn)-Ty/(r(nn)*mass(nn))
          w3(nn) = w3(nn)-Tz/(r(nn)*mass(nn))
                End If

C Collisions involving a boundary particle:

        else

C Collisions among boundary particles; only changes in velocity
C are considered:

                if (nh.gt.n) then

                v1(nn) = v1(nn)-Px/mass(nn)
                v2(nn) = 0.0
                v3(nn) = v3(nn)-Pz/mass(nn)

C No change in velocity of a wall particle when it is
C hit by an interior particle...:

                end if
        end if

      End If

CCCCCCCCCCCCCC
C
C Increment the statistics of particles colliding with the
C bottom wall (for details, see Jenkins, 1990):
C
CCCCCCCCCCCCCC

                if (nn.eq.nt+1) then

        ncol = ncol+1

C increment the number of no-slip collisions with the bottom wall:

        if (Pts .ge. Ptns) ncolns = ncolns + 1

C The counter of bottom wall collisions since the last MOMENTS
C output is incremented also:

        nstat = nstat + 1

C Increment the collisional impulse M:

CCCCC
C The following lines can be added to check changes with '0:
C
C        if (Pts .ge. Ptns) then
C       write(*,'(3(a,e12.6))')
C    &  'Px = ',Px,'Py = ',Py,'Pz = ',Pz
C       end if
C
CCCCC

        m1l = m1l + Px
        m2l = m2l + Py
        m3l = m3l + Pz

CCCCC
C In case the collision does not involve slip, increment the
C fraction of the impulse due to no-slip collisions:
C
          if (Pts .ge. Ptns) then

                m1lns = m1lns + Px
                m2lns = m2lns + Py
                m3lns = m3lns + Pz

          end if
C
CCCCC

C Increment the dissipation at the wall D. DTl is the contribution
C of translation energy to the dissipation at the lower wall. DRl
C is the contribution of rotational energy to the lower wall:

        DTl = DTl - mass(nh)/2.*
     &            ( v1(nh)**2 + v2(nh)**2 + v3(nh)**2 -
     &              v1b**2 - v2b**2 - v3b**2 ) +
     &              v1(nn)*Px + v2(nn)*Py + v3(nn)*Pz


        DRl = DRl - mass(nh)*r(nh)**2/5.*
     &            ( w1(nh)**2 + w2(nh)**2 + w3(nh)**2 -
     &              w1b**2 - w2b**2 - w3b**2 )

C Increment the absolute average velocity of particles
C colliding with the wall before the collision:

        u2l = u2l + v2b

C Increment the absolute velocity square of particles
C colliding with the wall before the collision:

        uu2l = uu2l + v2b*v2b

C Increment the absolute average velocity of particles
C colliding with the wall. Average the velocities before and
C after the collision:

        u1ltot = u1ltot + (v1(nh) + v1b)/2.
        u2ltot = u2ltot + (v2(nh) + v2b)/2.
        u3ltot = u3ltot + (v3(nh) + v3b)/2.

C Increment the absolute velocity square of particles
C colliding with the wall:

        uu1ltot = uu1ltot + (v1(nh)*v1(nh) + v1b*v1b)/2.
        uu2ltot = uu2ltot + (v2(nh)*v2(nh) + v2b*v2b)/2.
        uu3ltot = uu3ltot + (v3(nh)*v3(nh) + v3b*v3b)/2.

CCCCCCCCCCCCCCC
C Increment registers to calculate various collision integrals
C arising in the calculation of the flux of energy;
C du2, t1du2, t3du2, du2sq and dgtgn are fluxes with the correct
C sign. Typically, du2, t1du2 and dgtgn are <0; du2sq is >0:
C
        du2   = du2 + (          v2b )
        du1   = du1 + (          v1b )

C Note that the minus sign in front of t1 & t3 reflects the
C fact that the program calculates the vector (-t) in our
C notation:

        t1du2 = t1du2 + (-t1) * (          v2b )
        t3du2 = t3du2 + (-t3) * (          v2b )

        du2sq = du2sq + (             v2b**2 )
        du2cu = du2cu + (             v2b**3 )

        dgtgn = dgtgn +
     &          abs( (v1b    - v1(nn))*(-t1) +
     &                  v3b   *(-t3) +
     &        r(nh)*(w3b   *(-t1)-w1b   *(-t3)) )
     &                  *v2b

        g0a1 = g0a1 +
     &          ( v1(nh) - v1(nn) ) + r(nh)*w3(nh)
        g0a3 = g0a3 +
     &            v3(nh)            - r(nh)*w1(nh)

        g0b1 = g0b1 +
     &          ( v1b    - v1(nn) ) + r(nh)*w3b
        g0b3 = g0b3 +
     &            v3b               - r(nh)*w1b
     
     	gxgn = gxgn + (v1b - v1(nn) + r(nh)*w3b)*v2b

C
CCCCCCCCCCCCCCC

                end if

      Return
      End

*******************************************************************

      Subroutine Grdfil

C Note that the present version of this subroutine does not
C consider the flat walls as virtual particles.

        include "cou3fbbasccdpDcom.f"

      Integer*2 i,gi,gj,gk

      Do i = 1,Nt
         grid(invx(i),invy(i),invz(i)) = 0
      End do

C First, calculate the grid indices of interior particles:

      Do i = 1,N
        gi = int(x(i)/cell)+1
        gj = int(y(i)/cell)+2
        gk = int(z(i)/cell)+1
         grid(gi,gj,gk) = i
         invx(i) = gi
         invy(i) = gj
         invz(i) = gk
      End do

C Next, specify the grid indices of the bottom wall particles
C if they exist. Their index in the y-direction is 1:

        if (nl.ne.0) then

      Do i = N+1,NNl
        gi = int(x(i)/cell)+1
        gj = 1
        gk = int(z(i)/cell)+1
         grid(gi,gj,gk) = i
         invx(i) = gi
         invy(i) = gj
         invz(i) = gk
      End do

        end if

C Next, specify the grid indices of the top wall particles
C if they exist. Their index in the y-direction is NJ+2:

        if (nu.ne.0) then

      Do i = NNl+1,Nt
        gi = int(x(i)/cell)+1
        gj = nj+2
        gk = int(z(i)/cell)+1
         grid(gi,gj,gk) = i
         invx(i) = gi
         invy(i) = gj
         invz(i) = gk
      End do

        end if

      Return
      End

************************************************************

      Subroutine INITIAL

        include "cou3fbbasccdpDcom.f"

      Integer*2 i
      real*8 dummy,temp,factor,yoverdcrit

      Read (*,'(A)') fname(1)
      Read (*,'(A)') fname(2)
      Read (*,'(A)') fname(3)
      Read (*,'(A)') fname(4)
      Read (*,*) wallcoll
      read (*,*) duration
      Read (*,*) ncal
      Read (*,*) nsta
      Read (*,*) tolerance
      Read (*,*) nsearch
      Read (*,*) nstrip
      Read (*,*) flag
      Read (*,*) time0
      Read (*,*) yoverdcrit

      if (yoverdcrit.lt.2.0) then
        write (*,'(a)') 'Strip at y/d=2 unresolved'
        stop
      end if

      Read (*,*) ed
      Read (*,*) ew
      Read (*,*) mu
      Read (*,*) muw
      Read (*,*) beta0
      Read (*,*) beta0w
      Read (*,*) temp

C TEMP is input as the ratio of the relative slip between the walls
C and the square root of the 2-D granular temperature of the upper
C wall thermostat.

      Open(1, File=fname(1), Form='Formatted', Status='Old')

      read (1,'(3(10x,I5))') N,Nl,Nu
      read (1,'(3(10x,I5))') ni,nj,nk
C     read (1,'(2F15.10)') time0,dt
C     read (1,'(6F10.5)') ed,ew,mu,muw,beta0,beta0w
      read (1,'(2F15.10)') dummy,dt
      read (1,'(6F10.5)') dummy,dummy,dummy,dummy,dummy,dummy
      read (1,'(3F15.10)') Rmax,cell,con
      read (1,'(3F15.10)') sD,gravx,gravy
      read (1,'(3F15.10)') xlen,ylen,zlen

C     Spheres 1 thru N are interior spheres.
C     Spheres N+1 thru N+Nl are lower boundary spheres.
C     Spheres N+Nl+1 thru Nt are upper boundary spheres.
C     'Sphere' Nt+1 is the lower flat wall
C     'Sphere' Nt+2 is the upper flat wall

      NNl = N+Nl
      Nt = N+Nl+Nu
      Do i = 1, Nt+2
         read (1,'(4F15.10)') x(i),y(i),z(i),r(i)
         read (1,'(2(e19.13,2x),e19.13)') v1(i),v2(i),v3(i)
         read (1,'(2(e19.13,2x),e19.13)') w1(i),w2(i),w3(i)

         if (i .le. N) then

C Specify the mass of interior particles; assume that the
C density is one:

           aveD = aveD+r(i)
           mass(i) = 4.0*pi*r(i)*r(i)*r(i)/3.0

         else

C The mass of wall particles or the flat walls is very large:

             mass(i) = 1.0e08

         end if

      End do

      Close(1)

C Calculate the initial average velocity and temperature
C of the top wall particles:

        if (nu.ne.0) then

        ub1 = 0.0
        ub3 = 0.0
        Tb1 = 0.0
        Tb3 = 0.0

        do i = nnl+1,nt

        ub1 = ub1 + v1(i)
        ub3 = ub3 + v3(i)
        Tb1 = Tb1 + v1(i)*v1(i)
        Tb3 = Tb3 + v3(i)*v3(i)

        end do

        ub1 = ub1/nu
        ub3 = ub3/nu
        Tb1 = Tb1/nu - ub1**2
        Tb3 = Tb3/nu - ub3**2

CCC
C Rescale the velocity scale according to the desired temperature
C TEMP of the top wall as long as the initial temperature is large
C enough to do so:
C
                if (Tb1+Tb3.ge.1e-8) then

        if (temp.ne.0.0) temp = ((v1(Nt+2)-v1(Nt+1))/temp)**2

        factor = sqrt(temp/(Tb1+Tb3)*2.)

          do i = nnl+1,nt

        v1(i) = (v1(i)-ub1)*factor + ub1
        v3(i) = (v3(i)-ub3)*factor + ub3

          end do

                end if

C Save the new initial conditions of the top wall thermostat:

        Tb1 = Tb1*factor**2
        Tb3 = Tb3*factor**2

        end if

      time = time0
      aveD = 2.0*aveD/N
      strip = ylen/nstrip
      halfxlen = 0.5*xlen
      halfzlen = 0.5*zlen

C The following statement defines the largest strip index
C that is considered in the present program. This permits
C a precise determination of the strip statistics near the
C wall, without the need for extending the strips all the way
C to the thermostat. With the instruction
C
C       kcrit = int(3.*aveD/2./strip) + 1
C
C the strip located at y=3d/2 is resolved. Make sure that the
C value of kcrit is not greater than Ns, the dimension of the
C strip arrays in the COMMON statement. MYL 9/16/93.
C

      kcrit = int(yoverdcrit*aveD/strip) + 1

      if (kcrit.gt.Ns) then
        write (*,'(a)') 'Excessive number of strips'
        stop
      end if

      Return
      End

************************************************************

      Block Data InitVars

        include "cou3fbbasccdpDcom.f"

      Data Grid/dimijk*0/
      Data home,near/FNp*0,FNp*0/
      Data invx,invy,invz/Np*1,Np*1,Np*1/
      Data nco,ncol,ncalt,ncalp,ncalw,count,mcount,nstat/8*0/
      Data ncolns/0/
      Data aveD,over,overw,tpict,vnhnmx,pi/5*0.0,3.141593/
      Data sf,sfs,u1p,u2p,u3p,uu1p,uv12p,uu2p,uu3p,w1p,w2p,w3p
     &    /Ns*0.0,Ns*0.0,Ns*0.0,Ns*0.0,Ns*0.0,Ns*0.0,
     &     Ns*0.0,Ns*0.0,Ns*0.0,Ns*0.0,Ns*0.0,Ns*0.0/
      Data stickcol,slipcol /Ns*0,Ns*0/
      Data m1l,m2l,m3l,DTl,DRl,g1l,g2l,g3l,
     &     u1l,u2l,u3l,
     &     uu1l,uu2l,uu3l,
     &     u1ltot,u2ltot,u3ltot,
     &     uu1ltot,uu2ltot,uu3ltot,
     &     ww1l,ww2l,ww3l,
     &     w1l,w2l,w3l,
     &     m1lns,m2lns,m3lns /29*0.0/
      Data du2,t1du2,t3du2,du2sq,du2cu ,dgtgn,
     &     g0a1,g0a3,g0b1,g0b3,gxgn /11*0.0/


      End

*******************************************************************

      Subroutine Interaction

C     This subroutine loops through the near neighbor lists HOME/NEAR
C     it checks each pair of spheres for overlap and executes a
C     collision (calls Collide) if an overlap is found.

      include "cou3fbbasccdpDcom.f"

      Integer*2 nn,nh,i
      Real*8 D,dx,dy,dz,ds,n1,n2,n3,vnhn

      Do i = 1,count
        nh = home(i)
        nn = near(i)

C       calculate x/y/z components of the line of centers vector

        dx = x(nh)-x(nn)
        dy = y(nh)-y(nn)
        dz = z(nh)-z(nn)

C       periodic boundary conditions

        if (abs(dx) .gt.  halfxlen) dx = dx-sign(xlen,dx)
        if (abs(dz) .gt.  halfzlen) dz = dz-sign(zlen,dz)

C       in collisions with flat wall make n perpendicular to wall

        if (nn .gt. Nt) then
          dx = 0.0
          dz = 0.0
        end if

        ds = dx*dx+dy*dy+dz*dz
        D  = r(nh)+r(nn)

        if (D*D .gt. ds) then

            vnhn = dx*(v1(nn)-v1(nh))+dy*(v2(nn)-v2(nh))+
     &             dz*(v3(nn)-v3(nh))

C Note that, because dx=dz=0 and v2=0 for the flat wall,
C the above statement is equivalent to the following for
C flat walls:

C       vnhn = -dy*v2(nh)

          if (vnhn .gt. 0.0) then
            ds = sqrt(ds)
            n1 = dx/ds
            n2 = dy/ds
            n3 = dz/ds
            vnhn = vnhn/ds

            Call Collide(nn,nh,n1,n2,n3,dx,dy,dz,vnhn)
CCC
C If the particles collide with the wall, calculate the corresponding
C overlap by incrementing the registers OVERW and NCALW.
C If the collision occurs between two particles, increment
C OVER and NCALP instead:

                if (nn.ge.nt+1) then

            overw=overw+D-ds
            ncalw=ncalw+1

                else

            over = over+D-ds
            ncalp=ncalp+1

                end if
C
CCC
C Increment the total number of collisions since the last calibration NCALT:
C
            ncalt = ncalt+1
            nco = nco+1

          endif
        endif
      End Do

      Return
      End

********************************************************************

      subroutine Moments

C Simplified subroutine to calculate the particle
C volume fraction at y=d/2 and y=3d/2. MYL 6/12/1992.
C

        include "cou3fbbasccdpDcom.f"

      Real*8 uu1t,uu2t,uu3t,Area,Tarea,h
      Real*8 ww1t,ww2t,ww3t
      Integer*2 i,k

      nstat = 0
      mcount = mcount+1

C The following instructions consider that the particle contribute
C to a strip if and only if its center is located in that
C strip. MYL 9/17/93. Center statistics of volume fraction:

      Do 1 i = 1,N

         k = int(nstrip/ylen*y(i))+1

C Do not take into account particles with center
C located in a strip above the last strip
C considered here (k>kcrit):

                if (k.gt.kcrit) go to 1

           sf(k) = sf(k)+1.0

1      Continue

C The following instructions consider that the particle contribute
C to a strip if a portion of it is located in that
C strip. MYL 1/24/94. Strip statistics:

      Do 2 i = 1,N

         uu1t = v1(i)*v1(i)
         uu2t = v2(i)*v2(i)
         uu3t = v3(i)*v3(i)

         ww1t = w1(i)*w1(i)
         ww2t = w2(i)*w2(i)
         ww3t = w3(i)*w3(i)

         Area  = 0.0
         TArea = 0.0
         k = int(nstrip/ylen*(y(i)-r(i)))+1

C Do not take into account particles with bottom
C located in a strip above the last strip
C considered here (k>kcrit):

                if (k.gt.kcrit) go to 2

         h = k*strip-(y(i)-r(i))

         Do While (h .le. 2.0*r(i))

           Area = pi*h*h*(r(i)-h/3.0)-TArea

           sfs(k) = sfs(k)+Area

           u1p(k) = u1p(k)+Area*v1(i)
           u2p(k) = u2p(k)+Area*v2(i)
           u3p(k) = u3p(k)+Area*v3(i)

           w1p(k) = w1p(k)+Area*w1(i)
           w2p(k) = w2p(k)+Area*w2(i)
           w3p(k) = w3p(k)+Area*w3(i)

           uu1p(k) = uu1p(k)+Area*uu1t
           uu2p(k) = uu2p(k)+Area*uu2t
           uu3p(k) = uu3p(k)+Area*uu3t

           ww1p(k) = ww1p(k)+Area*ww1t
           ww2p(k) = ww2p(k)+Area*ww2t
           ww3p(k) = ww3p(k)+Area*ww3t

           TArea = TArea+Area
           k = k+1
           h = h+strip

C Do not take into account particles strip
C slices located above the last strip
C considered here (k>kcrit):

                if (k.gt.kcrit) go to 2

         End do

         Area = mass(i)-TArea

         sfs(k) = sfs(k)+Area

         u1p(k) = u1p(k)+Area*v1(i)
         u2p(k) = u2p(k)+Area*v2(i)
         u3p(k) = u3p(k)+Area*v3(i)

         w1p(k) = w1p(k)+Area*w1(i)
         w2p(k) = w2p(k)+Area*w2(i)
         w3p(k) = w3p(k)+Area*w3(i)

         uu1p(k) = uu1p(k)+Area*uu1t
         uu2p(k) = uu2p(k)+Area*uu2t
         uu3p(k) = uu3p(k)+Area*uu3t

         ww1p(k) = ww1p(k)+Area*ww1t
         ww2p(k) = ww2p(k)+Area*ww2t
         ww3p(k) = ww3p(k)+Area*ww3t

2      Continue

      Return
      End

*******************************************************************

      Subroutine MoveSpheres

        include "cou3fbbasccdpDcom.f"

      Integer*2 i
      Real*8 dx,dy

      time = time+dt

      dx = 0.5*dt*dt*gravx
      dy = 0.5*dt*dt*gravy

      Do i = 1, N
         x(i) = x(i)+dt*v1(i)+dx

C        periodic boundary conditions x direction

         if (x(i) .gt. xlen) x(i) = x(i)-xlen
         if (x(i) .lt. 0.0)  x(i) = x(i)+xlen

         y(i) = y(i)+dt*v2(i)+dy

C Ckeck for escaped particles:

        if (y(i).ge.ylen) then
        write (*,'(a,i4,a,/,6(a,f8.5))') 'Particle no ',i,
     &  ' has escaped from the upper wall',
     &      ' x =',x(i),' y =',y(i),
     &      ' z =',z(i),
     &      ' u =',v1(i),' v =',v2(i),
     &      ' w =',v3(i)
        stop
        end if

        if (y(i).le.0.0) then
        write (*,'(a,i4,a,/,6(a,f8.5))') 'Particle no ',i,
     &  ' has escaped from the lower wall',
     &      ' x =',x(i),' y =',y(i),
     &      ' z =',z(i),
     &      ' u =',v1(i),' v =',v2(i),
     &      ' w =',v3(i)
        stop
        end if

         z(i) = z(i)+dt*v3(i)

C        periodic boundary conditions z direction

         if (z(i) .gt. zlen) z(i) = z(i)-zlen
         if (z(i) .lt. 0.0)  z(i) = z(i)+zlen

         v1(i) = v1(i)+dt*gravx
         v2(i) = v2(i)+dt*gravy

      End do

C     Move the wall spheres if they exist (N+1 to Nt):

        if (Nl+Nu.ne.0) then

      Do i = N+1,Nt

         x(i) = x(i)+dt*v1(i)
         if (x(i) .gt. xlen) x(i) = x(i)-xlen
         if (x(i) .lt. 0.0) x(i) = x(i)+xlen

         z(i) = z(i)+dt*v3(i)
         if (z(i) .gt. zlen) z(i) = z(i)-zlen
         if (z(i) .lt. 0.0)  z(i) = z(i)+zlen

      End do

        end if

      Return
      End

************************************************************

      Subroutine Output

C Simplified subroutine to output the particle
C volume fraction at y=d/2. MYL 6/12/1992, 1/12/93, 2/8/93.

C       The outputs of this routine are
C       u/CT0, r, S/N, Q/NC3Ttr, N/rhoTtr(1+ew),
C       (nu ctr)1st peak,y/d 1st peak,(nu ctr)2nd peak,y/d 2nd peak,
C       nu strip 1st peak, nu average 1st layer,
C       (u-uwall)/C3Ttr @1st peak,(u-uwall)/C3Ttr @2nd peak,
C       (u-uwall)/C3Ttr @1st peak minus (u-uwall)/C3Ttr @2nd peak,
C       w/C3Ttr at wall,w/C3Ttr at 1st peak,w/C3Ttr at 2nd peak,
C
C               at first peak:
C
C       uu1/3Ttr,uu2/3Ttr,uu3/3Ttr,ww1/3Ttr,ww2/3Ttr,ww3/3Ttr,
C
C               at second peak:
C
C       uu1/3Ttr,uu2/3Ttr,uu3/3Ttr,ww1/3Ttr,ww2/3Ttr,ww3/3Ttr,
C
C       collision frequency with flat wall,Ttr/Ttot,
C       fraction of no-slip collisions with bottom wall,
C       corresponding fraction of the shear stress at the wall due
C       to no-slip collisions,N/rho uu2l (1+ew),uu2l/Ttr.

C	The additional dimensionless flux quantities are
C	F[|gt|gn|/d^3F[gn]F[1], F[t.x gn]/F[gn], F[t.z gn]/F[gn],
C	F[gn^2]/d^3F[gn]F[1],
C	g0after.x/d^3F[1],g0after.z/d^3F[1],
C	g0before.x/d^3F[1],g0before.z/d^3F[1],
C	g0ave.x/d^3F[1],g0ave.z/d^3F[1],
C	Dtr/m d^3F[gn]F[1],Drot/m d^3F[gn]F[1],
C	{F[|gt|gn]-F[t gn].g0after}/F[gn^2],
C	{F[|gt|gn]-F[t gn].g0before}/F[gn^2],
C	{F[|gt|gn]-F[t gn].g0ave}/F[gn^2],
C	Qmeas/mF[gn^2],Qtheory/mF[gn^2] with  term,
C	F[G.x g.n]/F[gn^2] with g0ave
C	approx equal to {F[|gt|gn]-F[t gn].g0ave}/F[gn^2].

C Note!!! The outputs are directly the ratio S/N and the flux,
C NOT R and FF!!

        include "cou3fbbasccdpDcom.f"

      Integer*2 i,kpeak,kpeak1,kpeak0
      Real*8 TTl,TRl,rl,temp
      Real*8 S1l,S2l,S3l,Ql,SNl
      Real*8 vr,v3r,wr2,w3r2
      Real*8 inertia,ystrip
      real*8 sfmax1,sfmax2
      real*8 ypeak1,ypeak2
      real*8 save
      real*8 dKEtra,dKErot
      real*8 mu0,diffgoa,diffgob,diffgoave

      integer*2 ilim1,ilim2,idiam,Ntyp

      Real*8 u1p0,u2p0,u3p0
      Real*8 w1p0,w2p0,w3p0
      real*8 w02,sf0
      Real*8 uu1p0,uu2p0,uu3p0
      Real*8 ww1p0,ww2p0,ww3p0

      Real*8 u1pkpeak,u2pkpeak,u3pkpeak
      Real*8 w1pkpeak,w2pkpeak,w3pkpeak
      Real*8 uu1pkpeak,uu2pkpeak,uu3pkpeak
      Real*8 ww1pkpeak,ww2pkpeak,ww3pkpeak

      Real*8 u1pkpeak1,u2pkpeak1,u3pkpeak1
      Real*8 w1pkpeak1,w2pkpeak1,w3pkpeak1
      Real*8 uu1pkpeak1,uu2pkpeak1,uu3pkpeak1
      Real*8 ww1pkpeak1,ww2pkpeak1,ww3pkpeak1

      Open(2, File=fname(2), Form='Formatted', Status='New')

      write (2,'(3I15)') N,Nl,Nu
      write (2,'(3I15)') ni,nj,nk
      write (2,'(2F15.10)') time,dt
      write (2,'(6F10.5)') ed,ew,mu,muw,beta0,beta0w
      write (2,'(3F15.10)') Rmax,cell,con
      write (2,'(3F15.10)') sD,gravx,gravy
      write (2,'(3F15.10)') xlen,ylen,zlen

      Do  i = 1, Nt+2
        write (2,'(4F15.10)') x(i),y(i),z(i),r(i)
        write (2,'(2(e19.13,2x),e19.13)') v1(i),v2(i),v3(i)
        write (2,'(2(e19.13,2x),e19.13)') w1(i),w2(i),w3(i)
      End do

      Close(2)

        duration = time - time0

        Ntyp = N/2

        dKEtra = DTl/mass(Ntyp)/aveD**3/(-du2)/
     &                  ncol*duration*xlen*zlen
        dKErot = DRl/mass(Ntyp)/aveD**3/(-du2)/
     &                  ncol*duration*xlen*zlen

        DTl = DTl / duration/xlen/zlen
        DRl = DRl / duration/xlen/zlen
        S1l = m1l/duration/xlen/zlen
        S2l = m2l/duration/xlen/zlen
        S3l = m3l/duration/xlen/zlen

CCC
C INERTIA is the moment of inertia per unit mass. It is used to calculate
C the rotational contribution to the total temperature.
CCC

      inertia = aveD**2/10.

CCC
C Calculate values at the wall by averaging over the first
C 12% of the sphere diameter:
CCC

          kpeak0 = int(0.12*aveD/strip) + 1

          sf0   =       0.0

          u1p0  =       0.0
          u2p0  =       0.0
          u3p0  =       0.0

          w1p0  =       0.0
          w2p0  =       0.0
          w3p0  =       0.0

          uu1p0 =       0.0
          uu2p0 =       0.0
          uu3p0 =       0.0

          ww1p0 =       0.0
          ww2p0 =       0.0
          ww3p0 =       0.0

                do i = 1,kpeak0

          sf0   =       sf0     +       sfs(i)

          u1p0  =       u1p0    +       u1p(i)
          u2p0  =       u2p0    +       u2p(i)
          u3p0  =       u3p0    +       u3p(i)

          w1p0  =       w1p0    +       w1p(i)
          w2p0  =       w2p0    +       w2p(i)
          w3p0  =       w3p0    +       w3p(i)

          uu1p0 =       uu1p0   +       uu1p(i)
          uu2p0 =       uu2p0   +       uu2p(i)
          uu3p0 =       uu3p0   +       uu3p(i)

          ww1p0 =       ww1p0   +       ww1p(i)
          ww2p0 =       ww2p0   +       ww2p(i)
          ww3p0 =       ww3p0   +       ww3p(i)

                end do

        if (sf0.gt.0.0) then

          u1p0 = u1p0/sf0
          u2p0 = u2p0/sf0
          u3p0 = u3p0/sf0

          w1p0 = w1p0/sf0
          w2p0 = w2p0/sf0
          w3p0 = w3p0/sf0

          uu1p0 = uu1p0/sf0-u1p0**2
          uu2p0 = uu2p0/sf0-u2p0**2
          uu3p0 = uu3p0/sf0-u3p0**2

          ww1p0 = ww1p0/sf0-w1p0**2
          ww2p0 = ww2p0/sf0-w2p0**2
          ww3p0 = ww3p0/sf0-w3p0**2

        end if

CCCCCCC
C
C Figure out the location of the first & second peaks;
C calculate the strip statistics:
C
CCCCCCC

        sfmax1 = 0.0
        sfmax2 = 0.0
        save   = 0.0

        idiam = int(aveD/strip) + 1

        ilim1 = int(aveD/strip+0.5)
        ilim2 = int(2*aveD/strip+1.0)

      Do i = 1,kcrit

        if (sfs(i).gt.0.0) then

          u1p(i) = u1p(i)/sfs(i)
          u2p(i) = u2p(i)/sfs(i)
          u3p(i) = u3p(i)/sfs(i)

          w1p(i) = w1p(i)/sfs(i)
          w2p(i) = w2p(i)/sfs(i)
          w3p(i) = w3p(i)/sfs(i)

          uu1p(i) = uu1p(i)/sfs(i)-u1p(i)**2
          uu2p(i) = uu2p(i)/sfs(i)-u2p(i)**2
          uu3p(i) = uu3p(i)/sfs(i)-u3p(i)**2

          ww1p(i) = (ww1p(i)/sfs(i)-w1p(i)**2)*inertia
          ww2p(i) = (ww2p(i)/sfs(i)-w2p(i)**2)*inertia
          ww3p(i) = (ww3p(i)/sfs(i)-w3p(i)**2)*inertia

          end if

        sfs(i) = sfs(i)/(mcount*xlen*zlen*strip)
        if (i.le.idiam) save = save + sfs(i)
        sf(i)  = sf(i)*Pi/6.*aveD**3/(mcount*xlen*zlen*strip)
C
C Look for the magnitude and location of the first and second
C peak of the center statistics of volume fraction:
C

        if (i.le.ilim1.and.sf(i).gt.sfmax1) then
                sfmax1  =       sf(i)
                kpeak   =       i
                ypeak1  =       strip*(i-0.5)/aveD
        end if

        if (i.le.ilim2.and.i.gt.ilim1.and.sf(i).gt.sfmax2) then
                sfmax2  =       sf(i)
                kpeak1  =       i
                ypeak2  =       strip*(i-0.5)/aveD
        end if

      End do

        save = save/float(idiam)

          u1pkpeak = u1p(kpeak)
          u2pkpeak = u2p(kpeak)
          u3pkpeak = u3p(kpeak)

          w1pkpeak = w1p(kpeak)
          w2pkpeak = w2p(kpeak)
          w3pkpeak = w3p(kpeak)

          uu1pkpeak = uu1p(kpeak)
          uu2pkpeak = uu2p(kpeak)
          uu3pkpeak = uu3p(kpeak)

          ww1pkpeak = ww1p(kpeak)
          ww2pkpeak = ww2p(kpeak)
          ww3pkpeak = ww3p(kpeak)

          u1pkpeak1 = u1p(kpeak1)
          u2pkpeak1 = u2p(kpeak1)
          u3pkpeak1 = u3p(kpeak1)

          w1pkpeak1 = w1p(kpeak1)
          w2pkpeak1 = w2p(kpeak1)
          w3pkpeak1 = w3p(kpeak1)

          uu1pkpeak1 = uu1p(kpeak1)
          uu2pkpeak1 = uu2p(kpeak1)
          uu3pkpeak1 = uu3p(kpeak1)

          ww1pkpeak1 = ww1p(kpeak1)
          ww2pkpeak1 = ww2p(kpeak1)
          ww3pkpeak1 = ww3p(kpeak1)

        g1l = ( u1p0 + w3p0*aveD/2. - v1(Nt+1) )
        g3l = ( u3p0 - w1p0*aveD/2. )

        TTl = ( uu1p0 + uu2p0 + uu3p0 )/3.
        TRl = ( ww1p0 + ww2p0 + ww3p0 )/3.*inertia

        vr = sqrt( (u1pkpeak - v1(Nt+1))**2 + u3pkpeak**2 )
        v3r = sqrt( (u1pkpeak1 - v1(Nt+1))**2 + u3pkpeak1**2 )

        w02  = w1p0 **2 + w3p0 **2
        wr2  = w1pkpeak **2 + w3pkpeak **2
        w3r2 = w1pkpeak1**2 + w3pkpeak1**2

        rl = sqrt( (g1l**2+g3l**2)/3./TTl )
        Ql = - g1l*S1l - g3l*S3l - DTl - DRl
        Ql = Ql/abs(S2l)/sqrt(3.*TTl)
        SNl = sqrt(s1l**2+s3l**2)/abs(s2l)

        u2l = u2l / float(ncol)
        uu2l = uu2l / float(ncol) - u2l**2

        u1ltot = u1ltot / float(ncol)
        uu1ltot = uu1ltot / float(ncol) - u1ltot**2

        u2ltot = u2ltot / float(ncol)
        uu2ltot = uu2ltot / float(ncol) - u2ltot**2

        u3ltot = u3ltot / float(ncol)
        uu3ltot = uu3ltot / float(ncol) - u3ltot**2

        temp = (uu1ltot + uu2ltot + uu3ltot)/3.

        write (*,'(2(f4.2,1HT),2(f6.2,1HT),2(f4.2,1HT),
     &  f4.2,1x,A24)')
     &  ed,ew,mu,muw,beta0,beta0w,ylen,programname
C
C Writing the global results on the output and profile data files:
C
        write (*,'(f6.2,1HT,
     &  e12.6,1HT,f7.4,1HT,f8.4,1HT,e11.6,1HR,/,
     &  f9.6,2(1HT,f6.4,1HT,f9.6),1HT,f8.6,1HR,/,
     &  e12.6,5(1HT,e12.6),1HR,/,e12.6,5(1HT,e12.6),1HR,/,
     &  e12.6,5(1HT,e12.6),1HR,/,e12.6,5(1HT,e12.6),1HR)')
     &  (v1(Nt+2)-v1(Nt+1))/sqrt((Tb1+Tb3)/2.),
     &  rl,SNl,Ql,abs(S2l)/TTl/(1.+ew),
     &  sfmax1,ypeak1,sfmax2,ypeak2,sfs(kpeak),save,
     &  vr/sqrt(3.*TTl),v3r/sqrt(3.*TTl),(v3r-vr)/sqrt(3.*TTl),
     &  aveD/2.*sqrt(w02/3./TTl),aveD/2.*sqrt(wr2/3./TTl),
     &  aveD/2.*sqrt(w3r2/3./TTl),
     &  uu1pkpeak/3./TTl,uu2pkpeak/3./TTl,uu3pkpeak/3./TTl,
     &  ww1pkpeak/3./TTl,ww2pkpeak/3./TTl,ww3pkpeak/3./TTl,
     &  uu1pkpeak1/3./TTl,uu2pkpeak1/3./TTl,uu3pkpeak1/3./TTl,
     &  ww1pkpeak1/3./TTl,ww2pkpeak1/3./TTl,ww3pkpeak1/3./TTl,
     &  ncol/duration/xlen/zlen*aveD**3/sqrt(TTl),
     &  TTl/(TTl+TRl),float(ncolns)/float(ncol),
     &  sqrt(m1lns**2+m3lns**2)/sqrt(m1l**2+m3l**2),
     &  abs(S2l)/uu2l/(1.+ew),uu2l/temp

	mu0 = 3.5*(1.+ew)*muw
	diffgoa   =
     &  (dgtgn-(t1du2*g0a1/ncol+t3du2*g0a3/ncol))/du2sq
	diffgob   =
     &  (dgtgn-(t1du2*g0b1/ncol+t3du2*g0b3/ncol))/du2sq
	diffgoave =
     &  (dgtgn-(t1du2*g1l+t3du2*g3l))/du2sq

        write (*,'(e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,2(1HT,e13.6))')
     &  dgtgn/aveD**3/du2/ncol*duration*xlen*zlen,
     &  t1du2/du2,t3du2/du2,
     &  du2sq/aveD**3/du2/ncol*duration*xlen*zlen,
     &  g0a1/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g0a3/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g0b1/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g0b3/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g1l/aveD**3/ncol*duration*xlen*zlen,
     &  g3l/aveD**3/ncol*duration*xlen*zlen,
     &  dKEtra,dKErot,diffgoa,diffgob,diffgoave,
     &	Ql*sqrt(3.*TTl)*(1.+ew)/(-du2sq/du2),
     &	0.5*(2./7.*mu0**2-(1.-ew**2))+2./7.*mu0*diffgoave,
     &	(gxgn-g1l*du2)/du2sq,g1l*ncol/du2,
     &	(du1-ncol*v1(Nt+1))/du2,g1l/sqrt(3.*TTl),
     &	(-du2cu)/du2sq**1.5,(-du2sq)/du2/sqrt(3.*TTl)

      Open(4, File=fname(4), Form='Formatted', Status='New')

        write (4,'(f6.2,1HT,
     &  e12.6,1HT,f7.4,1HT,f8.4,1HT,e11.6,1HR,/,
     &  f9.6,2(1HT,f6.4,1HT,f9.6),1HT,f8.6,1HR,/,
     &  e12.6,5(1HT,e12.6),1HR,/,e12.6,5(1HT,e12.6),1HR,/,
     &  e12.6,5(1HT,e12.6),1HR,/,e12.6,5(1HT,e12.6),1HR)')
     &  (v1(Nt+2)-v1(Nt+1))/sqrt((Tb1+Tb3)/2.),
     &  rl,SNl,Ql,abs(S2l)/TTl/(1.+ew),
     &  sfmax1,ypeak1,sfmax2,ypeak2,sfs(kpeak),save,
     &  vr/sqrt(3.*TTl),v3r/sqrt(3.*TTl),(v3r-vr)/sqrt(3.*TTl),
     &  aveD/2.*sqrt(w02/3./TTl),aveD/2.*sqrt(wr2/3./TTl),
     &  aveD/2.*sqrt(w3r2/3./TTl),
     &  uu1pkpeak/3./TTl,uu2pkpeak/3./TTl,uu3pkpeak/3./TTl,
     &  ww1pkpeak/3./TTl,ww2pkpeak/3./TTl,ww3pkpeak/3./TTl,
     &  uu1pkpeak1/3./TTl,uu2pkpeak1/3./TTl,uu3pkpeak1/3./TTl,
     &  ww1pkpeak1/3./TTl,ww2pkpeak1/3./TTl,ww3pkpeak1/3./TTl,
     &  ncol/duration/xlen/zlen*aveD**3/sqrt(TTl),
     &  TTl/(TTl+TRl),float(ncolns)/float(ncol),
     &  sqrt(m1lns**2+m3lns**2)/sqrt(m1l**2+m3l**2),
     &  abs(S2l)/uu2l/(1.+ew),uu2l/temp

        write (4,'(e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,4(1HT,e13.6),1HR,/,
     &             e13.6,2(1HT,e13.6))')
     &  dgtgn/aveD**3/du2/ncol*duration*xlen*zlen,
     &  t1du2/du2,t3du2/du2,
     &  du2sq/aveD**3/du2/ncol*duration*xlen*zlen,
     &  g0a1/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g0a3/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g0b1/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g0b3/ncol/aveD**3/ncol*duration*xlen*zlen,
     &  g1l/aveD**3/ncol*duration*xlen*zlen,
     &  g3l/aveD**3/ncol*duration*xlen*zlen,
     &  dKEtra,dKErot,diffgoa,diffgob,diffgoave,
     &	Ql*sqrt(3.*TTl)*(1.+ew)/(-du2sq/du2),
     &	0.5*(2./7.*mu0**2-(1.-ew**2))+2./7.*mu0*diffgoave,
     &	(gxgn-g1l*du2)/du2sq,g1l*ncol/du2,
     &	(du1-ncol*v1(Nt+1))/du2,g1l/sqrt(3.*TTl),
     &	(-du2cu)/du2sq**1.5,(-du2sq)/du2/sqrt(3.*TTl)

      write (4,'(i8)') kcrit
C
C Writing the dimensionless profiles.
C The velocity is relative to the flat plate and dimensionless
C with C3Ttr. The spin is made dimensionless with (d/2)/C3Ttr.
C The second moment of translational velocities is made dimensionless
C with 3Ttr.  The second moment of angular velocity fluctuations
C is made dimensionless with (d^2/10)/3Ttr. The vorticity is
C made dimensionless with (d/2)/C3Ttr.  MYL 12/20/93.
C
        write (4,104)

104     format (/,2x,1Hy,8x,4Hslip)

      Do i = 1,kcrit

        ystrip = strip*(i-0.5)/aveD

                if ((slipcol(i)+stickcol(i)).ne.0) then

        write (4,114) ystrip,
     &  float(slipcol(i))/(float(slipcol(i))+float(stickcol(i)))
                else

        write (4,124) ystrip

                end if

114     format (f6.4,1HT,e10.3)
124     format (f6.4)

      End do

        write (4,100)

100     format (/,2x,1Hy,8x,5H<SFc>,5x,5H<SFs>,4x,
     &            4H<u1>,8x,4H<u2>,6x,4H<u3>)

      Do i = 1,kcrit

        ystrip = strip*(i-0.5)/aveD

        write (4,110)   ystrip,sf(i),sfs(i),
     &                  (u1p(i)-v1(Nt+1))/sqrt(3.*TTl),
     &                  u2p(i)/sqrt(3.*TTl),
     &                  u3p(i)/sqrt(3.*TTl)

110     format (f6.4,1HT,4(e10.3,1HT),e10.3)

      End do

        write (4,101)

101     format (/,2x,1Hy,8x,4H<w1>,6x,4H<w2>,8x,4H<w3>)

      Do i = 1,kcrit

        ystrip = strip*(i-0.5)/aveD

        write (4,111)   ystrip,
     &                  aveD/2.*w1p(i)/sqrt(3.*TTl),
     &                  aveD/2.*w2p(i)/sqrt(3.*TTl),
     &                  aveD/2.*w3p(i)/sqrt(3.*TTl)

111     format (f6.4,1HT,2(e10.3,1HT),e10.3)

      End do

        write (4,102)

102     format (/,2x,1Hy,7x,5H<uu1>,5x,5H<uu2>,7x,5H<uu3>)

      Do i = 1,kcrit

        ystrip = strip*(i-0.5)/aveD

        write (4,112)   ystrip,
     &                  uu1p(i)/(3.*TTl),
     &                  uu2p(i)/(3.*TTl),
     &                  uu3p(i)/(3.*TTl)

112     format (f6.4,1HT,3(e10.3,1HT),e10.3)

      End do

        write (4,103)

103     format (/,2x,1Hy,7x,5H<ww1>,5x,5H<ww2>,7x,5H<ww3>)

      Do i = 1,kcrit

        ystrip = strip*(i-0.5)/aveD

        write (4,113)   ystrip,
     &                  ww1p(i)/(3.*TTl),
     &                  ww2p(i)/(3.*TTl),
     &                  ww3p(i)/(3.*TTl)

113     format (f6.4,1HT,2(e10.3,1HT),e10.3)

      End do

CCCC
C print half the vorticity in the z direction:
CCCC

        write (4,'(/,2x,1Hy,7x,4Hvor3,/)')

        do i = 1,kcrit-1

        write (4,'(f6.4,1HT,e10.3)')
     &  strip*float(i)/aveD,
     &  -aveD/2.*(u1p(i+1)-u1p(i))/2./strip/sqrt(3.*TTl)

        end do

      Close(4)

      Return
      End

********************************************************************

      Subroutine Search

      include "cou3fbbasccdpDcom.f"

      Integer*2 i,ii,j,k,kk,nh,nn,minj,maxj

      count = 0

CCCCCCC
C minj is the smallest grid index reserved for the bottom wall particles:
CCC
C
      minj = 1
C
C Similarly, maxj is the grid index for the upper boundary spheres:
C
        maxj = nj+2
C
CCCCCCC

C Check proximity of interior particles to flat walls:

      Do nh = 1,N
        if (invy(nh) .lt. 4) then
          count = count+1
          home(count) = nh
          near(count) = Nt+1
        end if

        if (nj-invy(nh) .lt. 1) then
          count = count+1
          home(count) = nh
          near(count) = Nt+2
        end if
      End Do

C Find near neighbors of the interior or boundary particles:

      Do nh = 1,Nt
        Do j = max(minj,invy(nh)-2),min(invy(nh)+2,maxj)

          Do i = invx(nh)-2,invx(nh)+2
            ii = i
            if (ii .gt. ni) ii = ii-ni
            if (ii .lt.  1) ii = ii+ni

            Do k = invz(nh)-2,invz(nh)+2
              kk = k
              if (kk .gt. nk) kk = kk-nk
              if (kk .lt.  1) kk = kk+nk

              nn = grid(ii,j,kk)

C Because the index of boundary particles is larger than that of interior
C particles, the boundary particles can only become "near" particles in
C their collisions with interior particles. However, boundary particles
C will become home particles in collisions with other boundary particles:

              if (nn .gt. nh) then
                count = count+1
                home(count) = nh
                near(count) = nn
              endif
            End Do
          End Do
        End Do

      End Do

        if(count.gt.FNp) then
        write (*,'(a,i10)') ' count = ',count,' exceeds FNp = ',FNp
        stop
        end if

        Return
      End

